<?php

# Copyright (c)  2008 - Marcus Lunzenauer <mlunzena@uos.de>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


require_once dirname(__FILE__).'/lib/tracxmlrpc.php';


require_once 'lib/functions.php';
require_once 'lib/classes/Config.class.php';


/**
 * Ein Standardplugin, dass Tickets aus einem Trac zeigt, und es ermöglicht,
 * weitere Tickets einzutragen.
 *
 * @package     studip
 * @subpackage  plugins
 *
 * @author    mlunzena
 * @copyright (c) Authors
 * @version   $Id: TracTickets.class.php 706 2008-05-27 22:09:01Z elmar $
 *
 * @license http://opensource.org/licenses/mit-license.php The MIT License
 */
class TracTickets extends AbstractStudIPStandardPlugin {


  /**
   * Contains the URL to your Trac.
   */
  private $trac_url;


  /**
   * This plugin's flexi template factory.
   */
  private $template_factory;


  /**
   * This plugin's cache instance.
   */
  private $cache;


  /**
   * Number of currently open tickets.
   */
  private $num_tickets;


  /**
   * Number of changed tickets since last visit.
   */
  private $changed_tickets;


  /**
   * Constructor.
   *
   * @return void
   */
  function __construct() {

    parent::__construct();


    // this plugin wants its own icon
    $this->setPluginiconname("public/images/bug.png");
    $this->setChangeIndicatorIconName("public/images/bug_red.png");


    // navigation
    $navigation = new StudipPluginNavigation();
    $navigation->setDisplayname('BIESTer');
    $navigation->setLink(PluginEngine::getLink($this));

    $mine = new StudipPluginNavigation();
    $mine->setDisplayname('Meine BIESTer');
    $mine->setLink(PluginEngine::getLink($this, array(), 'showMine'));
    $navigation->addSubmenu($mine);

    $add_bug = new StudipPluginNavigation();
    $add_bug->setDisplayname('BIEST melden');
    $add_bug->setLink(PluginEngine::getLink($this, array(), 'add'));
    $navigation->addSubmenu($add_bug);

    $this->setNavigation($navigation);
    $this->setShownInOverview(TRUE);

    $config = Config::GetInstance();
    $trac_url = $config->getValue('TRAC_URL');
    $trac_user = $config->getValue('TRAC_USER');
    $trac_password = $config->getValue('TRAC_PASSWORD');

    $this->trac_url = $trac_url;

    $this->template_factory =
      new Flexi_TemplateFactory(dirname(__FILE__).'/templates/');

    $this->cache = StudipCacheFactory::getCache();

    Trac::connect($trac_url . '/login/xmlrpc', $trac_user, $trac_password);
  }


  /**
   * Called when initializing the plugin.
   * Sets additional stylesheets to Stud.IP's layout.
   *
   * @return void
   */
  function initialize() {
    $css = $this->getPluginPath().'/public/stylesheets/application.css';

    $GLOBALS['_include_additional_header'] =
      '<link rel="stylesheet" href="'.$css.'" type="text/css">';
  }


  /**
   * Returns this plugin's name.
   *
   * @return string  the name of the plugin
   */
  function getPluginname() {
    return 'BIESTer';
  }


  /**
   * <MethodDescription>
   *
   * @return string  <description>
   */
  function getDisplaytitle() {
    return $this->getPluginname();
  }


  /**
   * <MethodDescription>
   *
   * @param  type       <description>
   *
   * @return type       <description>
   */
  function getPluginExistingItems($range_id) {
    return 0;
  }


  /**
   * <MethodDescription>
   *
   * @param  type       <description>
   *
   * @return type       <description>
   */
  function getOverviewMessage($has_changed) {
    $message = $this->num_tickets.' BIESTer';

    if ($has_changed) {
      $message .= ', '.$this->changed_tickets.' gendert';
    }

    return $message;
  }


  /**
   * <MethodDescription>
   *
   * @param  type       <description>
   *
   * @return type       <description>
   */
  function hasChanged($lastlogin) {
    $condition = 'status=new|assigned|reopened&type=BIEST&version=trunk';
    $cache_key = 'TracTickets/'.$condition;

    if (($_tickets = $this->cache->read($cache_key))) {
      $tickets = unserialize($_tickets);
    } else {
      $tickets = Trac_Ticket::find_all($condition);
      $this->cache->write($cache_key, serialize($tickets), 300);
    }

    $this->num_tickets = count($tickets);

    foreach ($tickets as $ticket) {
      if ($ticket->time_changed > $lastlogin) {
        ++$this->changed_tickets;
      }
    }

    return $this->changed_tickets > 0;
  }


  /**
   * Shows a listing of all tickets with matching fields.
   *
   * @param  array      ticket status or NULL (all open)
   * @param  string     ticket version or NULL (trunk)
   * @param  string     ticket owner or NULL (anyone)
   *
   * @return void
   */
  function showTicketList($status, $version, $owner) {

    if (!isset($status)) {
      $status = array('new', 'assigned', 'reopened');
    }

    if (!isset($version)) {
      $version = 'trunk';
    }

    $condition = 'status='.join('|', $status).'&type=BIEST';

    if ($version != 'all') {
      $condition .= '&version='.$version;
    }

    if (!empty($owner)) {
      $condition .= '&owner='.$owner;
    }

    if (($_versions = $this->cache->read('TracTickets/versions'))) {
      $versions = unserialize($_versions);
    } else {
      $versions = Trac_Ticket::versions();
      $this->cache->write('TracTickets/versions', serialize($versions));
    }

    array_unshift($versions, 'all');
    $tickets = Trac_Ticket::find_all($condition);
    asort($tickets);

    $last_visit = object_get_visit($this->getId(), 'sem');

    if ($last_visit === false) {
      $last_visit = 0;
    }

    echo $this->template_factory->render('all',
      array('plugin'     => $this,
            'status'     => $status,
            'version'    => $version,
            'owner'      => $owner,
            'versions'   => $versions,
            'tickets'    => $tickets,
            'last_visit' => $last_visit,
            'trac_url'   => $this->trac_url));

  }


  /**
   * Default action. Shows a listing of all tickets.
   *
   * @return void
   */
  function actionShow() {
    $this->showTicketList($_REQUEST['status'], $_REQUEST['version'], NULL);
  }


  /**
   * Shows a listing of all of your bugs.
   *
   * @return void
   */
  function actionShowMine() {
    $me = $GLOBALS['auth']->auth['uname'];
    $this->showTicketList($_REQUEST['status'], $_REQUEST['version'], $me);
  }


  /**
   * Shows and interprets a form to add a bug.
   *
   * @return void
   */
  function actionAdd() {

    if ($_SERVER["REQUEST_METHOD"] === 'POST') {

      $ticket = remove_magic_quotes($_POST['ticket']);

      $new = new Trac_Ticket();

      $new->summary     = $ticket['summary'];
      $new->description = $ticket['description'];
      $new->version     = $ticket['version'];
      $new->owner       = $ticket['owner'];

      $new->reporter    = $GLOBALS['auth']->auth['uname'];

      $new->type        = 'BIEST';
      $new->priority    = 'normal';
      $new->component   = 'Stud.IP allgemein';
      $new->severity    = 'normal';
      $new->status      = $ticket['owner'] ? 'assigned' : 'new';
      $new->save();

      $url = $GLOBALS['ABSOLUTE_URI_STUDIP'] . PluginEngine::getLink($this);
      header('Location: ' . $url);
      exit;
    }

    else {

      if (($_versions = $this->cache->read('TracTickets/versions'))) {
        $versions = unserialize($_versions);
      } else {
        $versions = Trac_Ticket::versions();
        $this->cache->write('TracTickets/versions', serialize($versions));
      }

      echo $this->template_factory->render('add',
        array('plugin'   => $this,
              'versions' => $versions));
    }
  }
}
