<?php

# Copyright (c)  2008 - Marcus Lunzenauer <mlunzena@uos.de>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


/**
 * The Trac_Ticket class exposes the Trac XML-RPC interface in an OO way.
 *
 * @package     plugins
 * @subpackage  trac_ticket
 *
 * @author    mlunzena
 * @copyright (c) Authors
 * @version   $Id: ticket.php 618 2008-04-03 09:50:54Z mlunzena $
 */

class Trac_Ticket {

  static private $fields = array(
    'id'           => array('type' => 'int'),
    'time_created' => array('type' => 'int'),
    'time_changed' => array('type' => 'int'),
    'summary'      => array('type' => 'string'),
    'description'  => array('type' => 'string'),
    'reporter'     => array('type' => 'string'),
    'owner'        => array('type' => 'string'),
    'type'         => array('type' => 'string'),
    'status'       => array('type' => 'string'),
    'priority'     => array('type' => 'string'),
    'milestone'    => array('type' => 'string'),
    'component'    => array('type' => 'string'),
    'version'      => array('type' => 'string'),
    'severity'     => array('type' => 'string'),
    'resolution'   => array('type' => 'string'),
    'keywords'     => array('type' => 'string'),
    'cc'           => array('type' => 'string'),
  );


  /**
   * Constructor.
   *
   * @param  array      an optional argument providing fields for the ticket.
   *
   * @return void
   */
  function __construct($attributes = array()) {
    $this->set_attributes($attributes);
  }


  /**
   * Returns a new Trac_Ticket object from an XML-RPC response.
   *
   * @param  array        the XML-RPC response
   *
   * @return Trac_Ticket  the newly created Trac_Ticket
   */
  static private function from_response($response) {
    list($id, $time_created, $time_changed, $attributes) = $response;
    return new Trac_Ticket(compact('id', 'time_created', 'time_changed') +
                           $attributes);
  }


  /**
   * Returns a new Trac_Ticket object from multiple ticket IDs.
   *
   * @param  array  an array of ticket IDs
   *
   * @return array  the newly created Trac_Tickets
   */
  static private function from_ids($ids) {
    $messages = array();
    foreach ($ids as $id) {
      $messages[] = array('ticket.get', $id);
    }

    $responses = Trac::connect()->send($messages);

    $result = array();
    foreach ($responses as $response) {
      $ticket = self::from_response($response);
      $result[$ticket->id] = $ticket;
    }
    return $result;
  }


  /**
   * Finds a ticket with TODO
   *
   * @param  type       <description>
   *
   * @return type       <description>
   */
  static function find($id) {
    return self::from_response(Trac::connect()->send('ticket.get', $id));
  }


  /**
   * <MethodDescription>
   *
   * @param  type       <description>
   *
   * @return type       <description>
   */
  static function find_all($condition = 'status!=closed') {
    $trac = Trac::connect();
    $ids = $trac->send('ticket.query', $condition);

    return self::from_ids($ids);
  }


  /**
   * <MethodDescription>
   *
   * @param  type       <description>
   *
   * @return type       <description>
   */
  static function recent_changes($date = NULL) {
    if (is_null($date)) {
      $date = time();
    }
    $trac = Trac::connect();
    $ids = $trac->send('ticket.getRecentChanges',
                       iso8601_encode($date, 1));

    return self::from_ids($ids);
  }


  static function milestones()  { return Trac::connect()->send('ticket.milestone.getAll'); }
  static function severities()  { return Trac::connect()->send('ticket.severity.getAll'); }
  static function resolutions() { return Trac::connect()->send('ticket.resolution.getAll'); }
  static function priorities()  { return Trac::connect()->send('ticket.priority.getAll'); }
  static function components()  { return Trac::connect()->send('ticket.component.getAll'); }
  static function versions()    { return Trac::connect()->send('ticket.version.getAll'); }
  static function statuses()    { return Trac::connect()->send('ticket.status.getAll'); }
  static function types()       { return Trac::connect()->send('ticket.type.getAll'); }


  /**
   * Sets all instance fields from the given array of attributes
   *
   * @param  array  an array of attributes
   *
   * @return void
   */
  private function set_attributes($attributes) {
    foreach ($attributes as $key => $value) {
      if (isset(self::$fields[$key])) {
        settype($value, self::$fields[$key]['type']);
        $this->$key = $value;
      }
    }
  }


  /**
   * <MethodDescription>
   *
   * @param  type       <description>
   *
   * @return type       <description>
   */
  function available_actions($id) {
    $trac = Trac::connect();
    $response = $trac->send('ticket.getAvailableActions', $id);
    var_dump($response);

  }


  /**
   * <MethodDescription>
   *
   * @param  type       <description>
   *
   * @return type       <description>
   */
  function save() {

    $attributes = array();
    foreach (self::$fields as $key => $value) {

      if (in_array($key, array('id', 'time_created', 'time_changed'))) {
        continue;
      }

      $attributes[$key] = @$this->$key;
      settype($attributes[$key], self::$fields[$key]['type']);
    }

    if (isset($this->id)) {
      $response = Trac::connect()->send('ticket.update', $this->id,
                                        $this->comment, $attributes, TRUE);
    }
    else {
      unset($attributes['summary'], $attributes['description']);
      $id = Trac::connect()->send('ticket.create', $this->summary,
                                  $this->description, $attributes, TRUE);
      $response = Trac::connect()->send('ticket.get', $id);
    }

    list($id, $time_created, $time_changed, $attributes) = $response;
    $attributes = compact('id', 'time_created', 'time_changed') + $attributes;
    $this->set_attributes($attributes);
  }


  /**
   * <MethodDescription>
   *
   * @return type       <description>
   */
  function delete() {
    $response = Trac::connect()->send('ticket.delete', $this->id);
    return $response;
  }
}
